#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from cgi import FieldStorage

# import interfaces
from pyams_file.interfaces import IFileField, IFileWidget, IImageField, IImageWidget, \
    IThumbnailImageWidget, IThumbnailImageField, DELETED_FILE
from pyams_form.interfaces.form import IFormLayer
from z3c.form.interfaces import NOT_CHANGED, IFieldWidget, IDataConverter

# import packages
from pyams_form.widget import widgettemplate_config
from pyams_utils.adapter import adapter_config
from z3c.form.browser.file import FileWidget as BaseFileWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.util import toBytes
from z3c.form.widget import FieldWidget
from zope.interface import implementer_only


@adapter_config(context=(IFileField, IFileWidget), provides=IDataConverter)
class FileUploadDataConverter(BaseDataConverter):
    """File upload data converter"""

    def toWidgetValue(self, value):
        return value

    def toFieldValue(self, value):
        deleted_field_name = '{0}__deleted'.format(self.widget.name)
        deleted = self.widget.request.params.get(deleted_field_name)
        if deleted:
            return DELETED_FILE
        if (value is None) or (value is NOT_CHANGED) or (value == ''):
            return NOT_CHANGED
        elif isinstance(value, FieldStorage):
            return value.filename, value.file
        elif isinstance(value, tuple):
            return value
        else:
            return toBytes(value)


@widgettemplate_config(mode='input', template='templates/file-input.pt', widget=IFileWidget, layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/file-display.pt', widget=IFileWidget, layer=IFormLayer)
@implementer_only(IFileWidget)
class FileWidget(BaseFileWidget):
    """File widget"""

    @property
    def current_value(self):
        if self.form.ignoreContext:
            return None
        lang = getattr(self, 'lang', None)
        if lang is None:
            return self.field.get(self.context)
        else:
            value = self.field.get(self.context)
            if isinstance(value, dict):
                value = value.get(lang)
            return value

    @property
    def deletable(self):
        if self.required:
            return False
        if not self.ignoreContext:
            value = self.current_value
        else:
            value = self.value
        return bool(value)


@adapter_config(context=(IFileField, IFormLayer), provides=IFieldWidget)
def FileFieldWidget(field, request):
    return FieldWidget(field, FileWidget(request))


@widgettemplate_config(mode='input', template='templates/image-input.pt', widget=IImageWidget, layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/image-display.pt', widget=IImageWidget, layer=IFormLayer)
@implementer_only(IImageWidget)
class ImageWidget(FileWidget):
    """Image widget"""


@adapter_config(context=(IImageField, IFormLayer), provides=IFieldWidget)
def ImageFieldWidget(field, request):
    return FieldWidget(field, ImageWidget(request))


@implementer_only(IThumbnailImageWidget)
class ThumbnailImageWidget(ImageWidget):
    """Image widget with thumbnail images selection"""


@adapter_config(context=(IThumbnailImageField, IFormLayer), provides=IFieldWidget)
def ThumbnailImageFieldWidget(field, request):
    return FieldWidget(field, ThumbnailImageWidget(request))
